from distutils.core import setup
setup(name='clicarriots',
      version='0.1',
      description = 'The unofficial simple client for carriots platform (in progress) (python 2.x/3.x)',
      author='Samuel de Ancos',
      author_email='sdeancos@gmail.com',
      url = 'https://github.com/sdeancos/clicarriots',
      download_url = 'https://github.com/sdeancos/clicarriots/tarball/0.1',
      packages=['clicarriots'],
      )
